#!/usr/bin/env python

import logging
import sys
from os.path import dirname

import django
from django.conf import settings
from django.test.runner import DiscoverRunner

sys.path.insert(0, dirname(__file__))

installed_apps = [
    "django.contrib.contenttypes",
    "django.contrib.auth",
    "django.contrib.sessions",
    "django.contrib.admin",
    "django.contrib.messages",
    "django_audit_events",
    "django_audit_events.tests",
    "django_audit_events.tests.utils",
]


class DisableMigrations:
    def __contains__(self, item):
        return True

    def __getitem__(self, item):
        return None


DEFAULT_SETTINGS = dict(
    ALLOWED_HOSTS=["localhost"],
    ROOT_URLCONF="django_audit_events.tests.urls",
    INSTALLED_APPS=installed_apps,
    DATABASES={
        "default": {
            "ENGINE": "django.db.backends.postgresql",
            "NAME": "postgres",
            "USER": "postgres",
            "PASSWORD": "postgres",
            "HOST": "localhost",
            "PORT": "5432",
        },
        "other": {"ENGINE": "django.db.backends.sqlite3"},
    },
    MIGRATION_MODULES=DisableMigrations(),
    TEMPLATES=[
        {
            "BACKEND": "django.template.backends.django.DjangoTemplates",
            "APP_DIRS": True,
            "OPTIONS": {
                "context_processors": [
                    "django.contrib.auth.context_processors.auth",
                    "django.contrib.messages.context_processors.messages",
                ]
            },
        }
    ],
)
MIDDLEWARE = [
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django_audit_events.middleware.AuditContextMiddleware",
]

if django.__version__ >= "2.0":
    DEFAULT_SETTINGS["MIDDLEWARE"] = MIDDLEWARE
else:
    DEFAULT_SETTINGS["MIDDLEWARE_CLASSES"] = MIDDLEWARE


def main():
    if not settings.configured:
        settings.configure(**DEFAULT_SETTINGS)
    django.setup()
    failures = DiscoverRunner(failfast=False).run_tests(["django_audit_events.tests"])
    sys.exit(failures)


if __name__ == "__main__":
    logging.basicConfig()
    main()



from django.db import IntegrityError
from django.http import QueryDict
from django.test import TestCase, override_settings

from django_audit_events.context import AuditContext
from django_audit_events.models import AuditEvent
from django_audit_events.tests.test_middleware import MiddlewareTestMixin
from django_audit_events.tests.utils.models import Poll


class TestModelMixin(TestCase):
    def setUp(self):
        self.poll = Poll.objects.create(
            question="Is it possible to create an audit event easily?"
        )


class AuditEventModelTestCase(TestModelMixin, MiddlewareTestMixin):
    def test_require_content_object(self):
        context = AuditContext()
        event = AuditEvent.from_context(context)
        self.assertIsInstance(event, AuditEvent)
        with self.assertRaises(IntegrityError):
            event.save()

    def test_create_from_empty_context(self):
        context = AuditContext()
        event = AuditEvent.from_context(context)
        self.assertIsInstance(event, AuditEvent)

        event.content_object = self.poll

        # Event is not saved to the DB yet.
        # noinspection PyTypeChecker
        with self.assertRaises(AuditEvent.DoesNotExist):
            AuditEvent.objects.get(pk=event.pk)

        event.save()
        # Assert fetching from DB not raises DoesNotExist exception
        AuditEvent.objects.get(pk=event.pk)

    @override_settings(AUDIT_INCLUDE_QUERY_PARAMS=True)
    def test_create_from_context(self):
        self.reset_request()
        self.request.GET = QueryDict("foo=bar")
        self.apply_middleware()

        event = AuditEvent.from_context(self.request.audit_context)
        self.assertIsInstance(event, AuditEvent)
        self.assertIn("foo", event.query_params)
        self.assertEqual(
            self.request.audit_context.query_params["foo"], event.query_params["foo"]
        )

        event.content_object = self.poll
        event.save()
        # Assert fetching from DB not raises DoesNotExist exception
        AuditEvent.objects.get(pk=event.pk)



# Generated by Django 3.0.11 on 2022-02-15 07:56

from django.db import migrations

try:
    from django.db.models import JSONField
except ImportError:
    from django.contrib.postgres.fields.jsonb import JSONField


class Migration(migrations.Migration):

    dependencies = [
        ('django_audit_events', '0004_auto_20210127_2021'),
    ]

    operations = [
        migrations.AddField(
            model_name='archivedauditevent',
            name='headers',
            field=JSONField(blank=True, default=dict, null=True, verbose_name='Headers'),
        ),
        migrations.AddField(
            model_name='auditevent',
            name='headers',
            field=JSONField(blank=True, default=dict, null=True, verbose_name='Headers'),
        ),
    ]



from django.contrib.auth.models import User
from rest_framework import serializers


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = ("pk", "email", "first_name", "last_name")


class EventSerializer(serializers.ModelSerializer):
    user = UserSerializer()

    class Meta:
        fields = "__all__"
        model = None



from copy import deepcopy

from django.apps import apps
from django.contrib.auth.models import AnonymousUser
from django.utils.module_loading import import_string

from django_audit_events.conf import settings

MASK_VALUE = "*" * 16


class AuditContext(object):
    extra_data = {}
    __slots__ = ["url", "remote_addr", "user", "query_params", "post_data", "headers"]

    def __init__(
        self,
        url=None,
        remote_addr=None,
        user=None,
        query_params=None,
        post_data=None,
        headers=None,
    ):
        super(AuditContext, self).__setattr__("url", url)
        super(AuditContext, self).__setattr__("remote_addr", remote_addr)
        super(AuditContext, self).__setattr__("user", user)
        super(AuditContext, self).__setattr__("query_params", query_params or {})
        super(AuditContext, self).__setattr__("post_data", post_data or {})
        super(AuditContext, self).__setattr__("headers", headers or {})

    def __setattr__(self, key, value):
        raise NotImplementedError

    def __delattr__(self, item):
        raise NotImplementedError

    @classmethod
    def from_request(cls, request):
        """
        Create a context from HTTP request

        :type request: django.http.HttpRequest
        :return: An audit context
        :rtype: AuditContext
        """
        # noinspection PyUnresolvedReferences
        client_ip_function = cls.get_client_ip_function()
        context = {
            "url": request.build_absolute_uri(),
            "remote_addr": client_ip_function(request),
            "user": request.user,
        }

        if isinstance(context["user"], AnonymousUser):
            context["user"] = None

        if settings.AUDIT_INCLUDE_QUERY_PARAMS:
            context["query_params"] = request.GET

        if settings.AUDIT_INCLUDE_POST_DATA:
            # Copy POST data before masking sensitive fields
            # noinspection PyArgumentList
            context["post_data"] = cls.mask_fields(
                request.POST.copy(),
                settings.AUDIT_MASK_POST_FIELDS,
            )

        if settings.AUDIT_INCLUDE_HEADERS and isinstance(
            settings.AUDIT_INCLUDE_HEADERS, list
        ):
            context["headers"] = {
                header.replace("HTTP_", "", 1)
                .replace("_", "-")
                .title(): request.META.get(header)
                for header in settings.AUDIT_INCLUDE_HEADERS
                if request.META.get(header)
            }

        return cls(**context)

    @staticmethod
    def mask_fields(data, fields_to_mask):
        """
        Replace sensitive information in POST data
        :param data: POST data
        :type data: dict|django.http.request.QueryDict
        :param fields_to_mask: Names of fields to mask
        :type fields_to_mask: tuple|list
        :return: Data with masked sensitive information
        :rtype: dict
        """
        for field in filter(lambda x: x in data, fields_to_mask):
            data[field] = MASK_VALUE
        return data

    def new_event(self):
        """
        :return: Unsaved audit event
        :rtype: django_audit_events.models.AbstractAuditEvent
        """
        model = apps.get_model(settings.AUDIT_EVENT_MODEL)
        event = model.from_context(self)
        event.content = deepcopy(self.extra_data)
        return event

    def create_event(self, content_object, **content):
        """
        :param content_object: Object for the audit event
        :param content: Event content
        :return: Saved audit event
        :rtype: django_audit_events.models.AbstractAuditEvent
        """
        event = self.new_event()
        event.content_object = content_object
        event.content.update(content)
        event.save()
        return event

    def create_fields_event(self, content_object, *fields, **content):
        """
        :param content_object: Object for the audit event
        :param fields: Fields to store in the event
        :param content: Event content
        :return: Saved audit event
        :rtype: django_audit_events.models.AbstractAuditEvent
        """
        for field in fields:
            content[field] = getattr(content_object, field)
        return self.create_event(content_object, **content)

    @classmethod
    def get_client_ip_function(cls):
        _function = settings.AUDIT_CLIENT_IP_RESOLVER_FUNCTION
        if callable(_function):
            return _function
        return import_string(_function)



from rest_framework_filters import FilterSet, filters

from django_audit_events.models import get_audit_event_model

AuditEvent = get_audit_event_model()


class AuditEventFilterSet(FilterSet):
    user = filters.NumberFilter(name="user")
    content_type = filters.NumberFilter(name="content_type")
    object_id = filters.NumberFilter(name="object_id")

    class Meta:
        model = AuditEvent
        fields = ("user", "content_type", "object_id")



import random
from datetime import timedelta

from django.db.models import Q
from django.test import TestCase, override_settings
from django.utils import timezone
from model_bakery import baker

from django_audit_events.models import ArchivedAuditEvent, AuditEvent
from django_audit_events.tasks import archive_old_audit_events
from django_audit_events.tests.utils.models import Poll


class ArchiveAuditEventTaskTestCase(TestCase):
    def setUp(self):
        events = [
            baker.prepare(AuditEvent, content_object=baker.make(Poll))
            for _ in range(100)
        ]
        AuditEvent.objects.bulk_create(events)
        for event in events:
            event.timestamp = self.get_random_past_date()
            event.save()

        for _ in range(25):
            random_event = random.choice(events)

            if random_event.content_object:
                random_event.content_object.delete()

            random_event.save()

    def tearDown(self):
        AuditEvent.objects.all().delete()
        ArchivedAuditEvent.objects.all().delete()

    @override_settings(CELERY_ALWAYS_EAGER=True)
    def test_archiver_task(self):
        older_than = 90
        query = Q(timestamp__lte=timezone.now() - timedelta(days=older_than))
        events_to_delete = set()

        for event in AuditEvent.objects.iterator():
            if event.content_object is None:
                events_to_delete.add(event.uuid)

        events_to_archive = AuditEvent.objects.filter(query).exclude(
            uuid__in=events_to_delete
        )
        uuids = set([e.uuid for e in events_to_archive])

        archive_old_audit_events(older_than=older_than)

        archived_events = ArchivedAuditEvent.objects.all()
        archived_uuids = set([e.uuid for e in archived_events])
        self.assertEqual(len(events_to_archive), len(archived_events))
        self.assertSetEqual(uuids, archived_uuids)
        self.assertFalse(AuditEvent.objects.filter(uuid__in=archived_uuids).exists())
        self.assertFalse(
            ArchivedAuditEvent.objects.filter(uuid__in=events_to_delete).exists()
        )
        self.assertFalse(AuditEvent.objects.filter(uuid__in=events_to_delete).exists())

    def get_random_past_date(self, base=timezone.now, date_range=timedelta(days=365)):
        if callable(base):
            base = base()
        seconds = date_range.total_seconds()
        minus_seconds = random.randint(0, seconds)
        return base - timedelta(seconds=minus_seconds)



# -*- coding: utf-8 -*-
"""Log audit events in Django"""

from __future__ import unicode_literals

try:
    from django_audit_events.version import __version__
except ImportError:
    __version__ = "0.1"

__author__ = "Onur Güzel"
__license__ = "MIT"
__maintainer__ = "Akinon"
__email__ = "dev@akinon.com"

default_app_config = "django_audit_events.apps.DjangoAuditEventsConfig"



from django.test import TestCase

from django_audit_events.conf import DEFAULT_SETTINGS, settings


class SettingsTestCase(TestCase):
    def test_settings_has_defaults(self):
        self.assertEqual(
            settings.AUDIT_EVENT_MODEL, DEFAULT_SETTINGS["AUDIT_EVENT_MODEL"]
        )



from django.conf import settings
from django.utils.deprecation import MiddlewareMixin
from django.utils.functional import SimpleLazyObject

from django_audit_events.context import AuditContext


class AuditEventsMiddleware(MiddlewareMixin):
    # noinspection PyMethodMayBeStatic
    def process_request(self, request):
        assert hasattr(request, "user"), (
            "Audit Events middleware requires authentication middleware "
            "to be installed. Edit your MIDDLEWARE{} setting to insert "
            "'django.contrib.auth.middleware.AuthenticationMiddleware' before "
            "'django_audit_events.middleware.AuditEventsMiddleware'."
        ).format("_CLASSES" if settings.MIDDLEWARE is None else "")
        request.audit_context = SimpleLazyObject(
            lambda: AuditContext.from_request(request)
        )



from __future__ import unicode_literals

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class DjangoAuditEventsConfig(AppConfig):
    name = "django_audit_events"
    verbose_name = _("Django Audit Event")



from django.test import TestCase

from django_audit_events.context import AuditContext
from django_audit_events.mixin import AuditContextMixin


class TestClass(AuditContextMixin):
    dummy_attribute = "Dummy"


class TestWithDunder(AuditContextMixin):
    dummy_attribute = "Dummy"

    def __setattr__(self, key, value):
        setattr(self, key, value)


class MixinTestCase(TestCase):
    def test_has_audit_context(self):
        instance = TestClass()
        self.assertTrue(hasattr(instance, "audit_context"))

    def test_set_audit_context(self):
        instance = TestClass()
        context = AuditContext()
        instance.audit_context = context
        self.assertTrue(hasattr(instance, "audit_context"))

    def test_set_audit_context_none(self):
        instance = TestClass()
        with self.assertRaises(ValueError):
            instance.audit_context = None

    def test_set_audit_context_noninstance(self):
        instance = TestClass()
        with self.assertRaises(TypeError):
            instance.audit_context = "Not Acceptable"

    # noinspection PyMethodMayBeStatic
    def test_set_another_existing_attribute(self):
        instance = TestClass()
        instance.dummy_attribute = "Acceptable"

    def test_with_dunder_setattr(self):
        instance = TestClass()
        with self.assertRaises(ValueError):
            instance.audit_context = None

        with self.assertRaises(TypeError):
            instance.audit_context = "Not Acceptable"

        instance.dummy_attribute = "Acceptable"



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from django_audit_events.context import AuditContext


class AuditContextMixin(object):
    audit_context = AuditContext()

    def __setattr__(self, key, value):
        if key != "audit_context":
            return super(AuditContextMixin, self).__setattr__(key, value)

        if value is None:
            raise ValueError(_("Audit context cannot be None"))

        if not isinstance(value, AuditContext):
            raise TypeError(_("Audit context must be an AuditContext instance"))

        return super(AuditContextMixin, self).__setattr__(key, value)



from __future__ import unicode_literals

import uuid

from django.apps import apps
from django.conf import settings as django_settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.utils.translation import ugettext_lazy as _

from django_audit_events.conf import settings

try:
    from django.db.models import JSONField
except ImportError:
    from django.contrib.postgres.fields import JSONField


class AbstractAuditEvent(models.Model):
    uuid = models.UUIDField(
        verbose_name=_("ID"),
        default=uuid.uuid4,
        primary_key=True,
    )
    timestamp = models.DateTimeField(verbose_name=_("Timestamp"), auto_now_add=True)
    user = models.ForeignKey(
        django_settings.AUTH_USER_MODEL,
        on_delete=models.SET_NULL,
        verbose_name=_("User"),
        blank=True,
        null=True,
    )
    remote_addr = models.GenericIPAddressField(
        verbose_name=_("IP address"),
        blank=True,
        null=True,
    )
    url = models.URLField(
        verbose_name=_("URL"),
        blank=True,
        null=True,
        max_length=1000,
    )
    query_params = JSONField(
        verbose_name=_("Query params"),
        default=dict,
        blank=True,
        null=True,
    )
    post_data = JSONField(
        verbose_name=_("Post data"),
        default=dict,
        blank=True,
        null=True,
    )
    headers = JSONField(
        verbose_name=_("Headers"),
        default=dict,
        blank=True,
        null=True,
    )
    content_type = models.ForeignKey(
        ContentType,
        on_delete=models.CASCADE,
    )
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey("content_type", "object_id")
    content = JSONField(
        verbose_name=_("Content"),
        default=dict,
    )

    class Meta:
        verbose_name = _("Audit event")
        verbose_name_plural = _("Audit events")
        ordering = ("-timestamp",)
        abstract = True

    @classmethod
    def from_context(cls, context):
        """
        Create an event from audit context

        :type context: django_audit_events.context.AuditContext
        :return: An audit event object
        :rtype: AbstractAuditEvent
        """

        obj = cls()
        obj.user = context.user
        obj.remote_addr = context.remote_addr
        obj.url = context.url
        obj.query_params = context.query_params
        obj.post_data = context.post_data
        obj.headers = context.headers
        return obj

    def as_archive_event(self):
        archive_model = get_audit_event_archive_model()
        fields = archive_model._meta.get_fields()
        return archive_model(
            **{field.name: getattr(self, field.name, None) for field in fields}
        )


class AbstractArchivedAuditEvent(AbstractAuditEvent):
    timestamp = models.DateTimeField(verbose_name=_("Timestamp"))
    archive_timestamp = models.DateTimeField(
        verbose_name=_("Archive timestamp"), auto_now_add=True
    )

    class Meta:
        verbose_name = _("Archived audit event")
        verbose_name_plural = _("Archived audit events")
        ordering = ("-timestamp",)
        abstract = True


class AuditEvent(AbstractAuditEvent):
    class Meta(AbstractAuditEvent.Meta):
        swappable = "AUDIT_EVENT_MODEL"


class ArchivedAuditEvent(AbstractArchivedAuditEvent):
    class Meta(AbstractArchivedAuditEvent.Meta):
        swappable = "AUDIT_EVENT_ARCHIVE_MODEL"


def get_audit_event_model():
    try:
        return apps.get_model(settings.AUDIT_EVENT_MODEL)
    except ValueError:
        raise ImproperlyConfigured(
            "AUDIT_EVENT_MODEL must be of the form 'app_label.model_name'"
        )
    except LookupError:
        raise ImproperlyConfigured(
            "AUDIT_EVENT_MODEL refers to model '%s' that has not been installed"
            % settings.AUDIT_EVENT_MODEL
        )


def get_audit_event_archive_model():
    try:
        return apps.get_model(settings.AUDIT_EVENT_ARCHIVE_MODEL)
    except ValueError:
        raise ImproperlyConfigured(
            "AUDIT_EVENT_ARCHIVE_MODEL must be of the form 'app_label.model_name'"
        )
    except LookupError:
        raise ImproperlyConfigured(
            "'%s' that has not been installed" % settings.AUDIT_EVENT_ARCHIVE_MODEL
        )



from django.contrib import admin

from django_audit_events.models import ArchivedAuditEvent, AuditEvent


@admin.register(AuditEvent)
class AuditEventAdmin(admin.ModelAdmin):
    list_display = ("uuid", "user", "content_object", "timestamp")


@admin.register(ArchivedAuditEvent)
class ArchivedAuditEventAdmin(admin.ModelAdmin):
    list_display = ("uuid", "user", "content_object", "timestamp")



# Generated by Django 3.0.1 on 2019-12-26 14:35

import uuid

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

try:
    from django.db.models import JSONField
except ImportError:
    from django.contrib.postgres.fields.jsonb import JSONField


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AuditEvent',
            fields=[
                ('uuid', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False, verbose_name='ID')),
                ('timestamp', models.DateTimeField(auto_now_add=True, verbose_name='Timestamp')),
                ('remote_addr', models.GenericIPAddressField(blank=True, null=True, verbose_name='IP address')),
                ('url', models.URLField(blank=True, null=True, verbose_name='URL')),
                ('query_params', JSONField(blank=True, default=dict, null=True, verbose_name='Query params')),
                ('post_data', JSONField(blank=True, default=dict, null=True, verbose_name='Post data')),
                ('object_id', models.PositiveIntegerField()),
                ('content', JSONField(default=dict, verbose_name='Content')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'ordering': ('-timestamp',),
                'verbose_name': 'Audit event',
                'verbose_name_plural': 'Audit events',
                'abstract': False,
                'swappable': 'AUDIT_EVENT_MODEL',
            },
        ),
    ]



import logging
from datetime import timedelta

from celery import current_app as app
from django.db import transaction
from django.utils import timezone

from django_audit_events.models import (
    get_audit_event_archive_model,
    get_audit_event_model,
)

logger = logging.getLogger(__name__)


def _chunks(items, chunks_size=200):
    index, _items = 0, items
    while _items:
        _items = items[index * chunks_size : (index + 1) * chunks_size]
        if _items:
            yield _items
        index += 1


@app.task
def archive_old_audit_events(older_than=90):
    audit_event_model = get_audit_event_model()
    archived_audit_event_model = get_audit_event_archive_model()

    reference_date = timezone.now() - timedelta(days=older_than)
    events_to_delete = set()

    for event in audit_event_model.objects.iterator():
        if event.content_object is None:
            events_to_delete.add(event.uuid)

    events_to_delete = list(events_to_delete)

    for chunk in _chunks(events_to_delete):
        with transaction.atomic():
            audit_event_model.objects.filter(uuid__in=[e for e in chunk]).delete()

    events_to_archive = audit_event_model.objects.filter(timestamp__lte=reference_date)
    archive_items = [event.as_archive_event() for event in events_to_archive]

    for chunk in _chunks(archive_items):
        with transaction.atomic():
            archived_audit_event_model.objects.bulk_create(chunk)
            audit_event_model.objects.filter(uuid__in=[e.uuid for e in chunk]).delete()

    logger.info("Archived %d audit events." % events_to_archive.count())
    logger.info("Deleted %d audit events." % len(events_to_delete))



# Generated by Django 3.0.11 on 2021-01-27 20:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_audit_events', '0003_auto_20201218_1113'),
    ]

    operations = [
        migrations.AlterField(
            model_name='archivedauditevent',
            name='url',
            field=models.URLField(blank=True, max_length=1000, null=True, verbose_name='URL'),
        ),
        migrations.AlterField(
            model_name='auditevent',
            name='url',
            field=models.URLField(blank=True, max_length=1000, null=True, verbose_name='URL'),
        ),
    ]



from django.contrib.auth.middleware import AuthenticationMiddleware
from django.contrib.sessions.middleware import SessionMiddleware
from django.http import HttpRequest
from django.test import TestCase

from django_audit_events.middleware import AuditEventsMiddleware


class MiddlewareTestMixin:
    request = HttpRequest()

    def reset_request(self):
        self.request = HttpRequest()
        self.request.META = {"SERVER_NAME": "localhost", "SERVER_PORT": 80}

    def apply_middleware(self):
        SessionMiddleware().process_request(self.request)
        AuthenticationMiddleware().process_request(self.request)
        AuditEventsMiddleware().process_request(self.request)


class MiddlewareTestCase(TestCase, MiddlewareTestMixin):
    def setUp(self):
        self.reset_request()

    def test_without_authentication_middleware(self):
        with self.assertRaises(AssertionError):
            AuditEventsMiddleware().process_request(self.request)

    def test_with_authentication_middleware(self):
        self.apply_middleware()
        self.assertTrue(hasattr(self.request, "audit_context"))
        # noinspection PyUnresolvedReferences
        self.assertIsNotNone(self.request.audit_context)



import os
import sys

from setuptools import find_packages, setup

# This should always be before importing django_audit_events
# We need working directory in sys.path for setuptools.build_meta
# else we get ImportError
sys.path.insert(0, os.path.dirname(__file__))

from django_audit_events import (  # isort:skip
    __author__,
    __doc__,
    __email__,
    __license__,
    __maintainer__,
)

if sys.version_info[0] == 2:
    from io import open


def get_long_description():
    with open("README.md", "r", encoding="utf-8") as fh:
        return fh.read()


setup(
    name="django-audit-events",
    use_scm_version={
        "write_to": "django_audit_events/version.py",
        "write_to_template": '__version__ = "{version}"\n',
    },
    description=__doc__,
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author=__author__,
    author_email=__email__,
    maintainer=__maintainer__,
    maintainer_email=__email__,
    license=__license__,
    url="https://bitbucket.org/akinonteam/django-audit-events",
    project_urls={
        "Documentation": "https://django-audit-events.readthedocs.io/",
        "Source Code": "https://bitbucket.org/akinonteam/django-audit-events",
    },
    packages=find_packages(exclude=["tests", "tests.*"]),
    platforms="any",
    zip_safe=False,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 1.10",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)



from django.db.models import ForeignObject, ManyToManyField
from rest_framework import viewsets

from django_audit_events.filters import AuditEventFilterSet
from django_audit_events.models import get_audit_event_model
from django_audit_events.serializers import EventSerializer

AuditEvent = get_audit_event_model()


class AuditEventViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = AuditEvent.objects.all()
    filter_class = AuditEventFilterSet

    def __new__(cls, *args, **kwargs):
        obj = super(AuditEventViewSet, cls).__new__(cls, *args, **kwargs)
        obj.filter_class.Meta.model = AuditEvent
        return obj

    # noinspection PyMethodMayBeStatic
    def get_serializer_class(self, *args, **kwargs):
        EventSerializer.Meta.model = AuditEvent
        return EventSerializer

    def get_queryset(self):
        # noinspection PyProtectedMember
        fields = AuditEvent._meta.local_fields
        select_related = []
        prefetch_related = []

        for field in fields:
            if isinstance(field, ForeignObject):
                select_related.append(field.name)
            elif isinstance(field, ManyToManyField):
                prefetch_related.append(field.name)

        queryset = super(AuditEventViewSet, self).get_queryset()
        return queryset.select_related(*select_related).prefetch_related(
            *prefetch_related
        )



from django.conf import settings as django_settings

DEFAULT_SETTINGS = {
    "AUDIT_EVENT_MODEL": "django_audit_events.AuditEvent",
    "AUDIT_EVENT_ARCHIVE_MODEL": "django_audit_events.ArchivedAuditEvent",
    "AUDIT_INCLUDE_QUERY_PARAMS": False,
    "AUDIT_INCLUDE_POST_DATA": False,
    "AUDIT_INCLUDE_HEADERS": False,
    "AUDIT_MASK_POST_FIELDS": ("password",),
    "AUDIT_CLIENT_IP_RESOLVER_FUNCTION": "django_audit_events.utils.get_client_ip",
}


class Settings(object):
    def __getattr__(self, item):
        if item in DEFAULT_SETTINGS:
            return getattr(django_settings, item, DEFAULT_SETTINGS[item])
        raise AttributeError


settings = Settings()



from django.core.exceptions import ValidationError
from django.core.validators import validate_ipv46_address
from django.http.request import split_domain_port


def get_client_ip(request):
    x_forwarded_for = request.META.get("HTTP_X_FORWARDED_FOR")
    if x_forwarded_for:
        for remote_addr in x_forwarded_for.split(","):
            try:
                remote_addr = remote_addr.strip()
                remote_addr, _ = split_domain_port(remote_addr)
                validate_ipv46_address(remote_addr)
                return remote_addr
            except (ValidationError, TypeError):
                continue

    remote_addr = request.META.get("REMOTE_ADDR", "")
    try:
        remote_addr = remote_addr.strip()
        remote_addr, _ = split_domain_port(remote_addr)
        validate_ipv46_address(remote_addr)
        return remote_addr
    except (ValidationError, TypeError):
        return None



# -*- coding: utf-8 -*-
# Generated by Django 1.10.3 on 2020-12-18 11:13
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_audit_events', '0002_archivedauditevent'),
    ]

    operations = [
        migrations.AlterField(
            model_name='archivedauditevent',
            name='url',
            field=models.URLField(blank=True, max_length=256, null=True, verbose_name='URL'),
        ),
        migrations.AlterField(
            model_name='auditevent',
            name='url',
            field=models.URLField(blank=True, max_length=256, null=True, verbose_name='URL'),
        ),
    ]



from copy import deepcopy

from django.http import QueryDict
from django.test import override_settings
from django.utils.http import urlencode

from django_audit_events.context import MASK_VALUE, AuditContext
from django_audit_events.models import AuditEvent
from django_audit_events.tests.test_middleware import MiddlewareTestMixin
from django_audit_events.tests.test_models import TestModelMixin
from django_audit_events.tests.utils import custom_client_ip_function

custom_client_ip_func = "django_audit_events.tests.utils.custom_client_ip_function"


class ContextTestCase(TestModelMixin, MiddlewareTestMixin):
    def setUp(self):
        self.reset_request()
        super(ContextTestCase, self).setUp()

    # noinspection PyMethodMayBeStatic
    def test_create_blank(self):
        # Assert creating blank context not raises any exceptions
        AuditContext()

    def test_create_from_request(self):
        self.apply_middleware()
        # Assert creating context from request not raises any exceptions
        AuditContext.from_request(self.request)

    @override_settings(AUDIT_INCLUDE_HEADERS=["SERVER_NAME"])
    def test_create_from_request_with_include_headers_setting_enabled(self):
        self.apply_middleware()
        # Assert creating context from request not raises any exceptions
        result = AuditContext.from_request(self.request)

        self.assertDictEqual(result.headers, {"Server-Name": "localhost"})

    def test_context_is_immutable(self):
        context = AuditContext()

        with self.assertRaises(NotImplementedError):
            context.url = ""

        with self.assertRaises(NotImplementedError):
            # noinspection PyDunderSlots,PyUnresolvedReferences
            context.extra_data = {}

    @override_settings(AUDIT_INCLUDE_POST_DATA=True)
    def test_mask_post_data_fields(self):
        # Test static method without audit context
        data = {"foo": "bar", "password": "secret"}
        masked = AuditContext.mask_fields(deepcopy(data), ["password"])
        self.assertEqual(masked["foo"], "bar")
        self.assertEqual(masked["password"], MASK_VALUE)
        self.assertEqual(data["password"], "secret")

        # Test inside audit context
        self.request.POST = QueryDict(query_string=urlencode(data))
        self.apply_middleware()
        self.assertIsNotNone(self.request.audit_context.post_data)
        self.assertEqual(self.request.audit_context.post_data["foo"], "bar")
        self.assertEqual(self.request.audit_context.post_data["password"], MASK_VALUE)

    def test_new_event(self):
        self.apply_middleware()
        event = self.request.audit_context.new_event()
        self.assertIsInstance(event, AuditEvent)

    def test_create_event(self):
        self.apply_middleware()

        event = self.request.audit_context.create_event(self.poll, foo="bar")
        self.assertIsInstance(event, AuditEvent)
        AuditEvent.objects.get(pk=event.pk)
        self.assertIn("foo", event.content)
        self.assertEqual(event.content["foo"], "bar")

    def test_create_event_with_extra(self):
        self.apply_middleware()
        self.request.audit_context.extra_data["marco"] = "polo"

        event = self.request.audit_context.create_event(self.poll, foo="bar")
        self.assertIsInstance(event, AuditEvent)
        AuditEvent.objects.get(pk=event.pk)
        self.assertIn("marco", event.content)
        self.assertEqual(event.content["marco"], "polo")
        self.assertIn("foo", event.content)
        self.assertEqual(event.content["foo"], "bar")

    def test_create_event_with_extra_override(self):
        self.apply_middleware()
        self.request.audit_context.extra_data["foo"] = "bar"

        event = self.request.audit_context.create_event(self.poll, foo="baz")
        self.assertIsInstance(event, AuditEvent)
        AuditEvent.objects.get(pk=event.pk)
        self.assertIn("foo", event.content)
        self.assertEqual(event.content["foo"], "baz")

    @override_settings(AUDIT_INCLUDE_HEADERS=["SERVER_NAME"])
    def test_create_event_with_include_headers_setting_enabled(self):
        self.apply_middleware()

        event = self.request.audit_context.create_event(self.poll)
        self.assertIsInstance(event, AuditEvent)
        AuditEvent.objects.get(pk=event.pk)
        self.assertIn("Server-Name", event.headers)

    def test_create_fields_event(self):
        self.apply_middleware()

        event = self.request.audit_context.create_fields_event(
            self.poll, "question", foo="bar"
        )
        self.assertIsInstance(event, AuditEvent)
        AuditEvent.objects.get(pk=event.pk)
        self.assertIn("foo", event.content)
        self.assertEqual(event.content["foo"], "bar")
        self.assertIn("question", event.content)
        self.assertEqual(event.content["question"], self.poll.question)

    def test_get_default_client_ip_function(self):
        self.apply_middleware()
        remote_addr = "192.168.1.1"
        self.request.META["REMOTE_ADDR"] = remote_addr
        result = AuditContext.from_request(self.request)
        self.assertEqual(result.remote_addr, remote_addr)

    @override_settings(AUDIT_CLIENT_IP_RESOLVER_FUNCTION=custom_client_ip_function)
    def test_get_custom_client_ip_function(self):
        self.apply_middleware()
        remote_addr = "192.168.1.1"
        changed_remote_addr = "192.168.2.1"
        self.request.META["REMOTE_ADDR"] = remote_addr
        result = AuditContext.from_request(self.request)
        self.assertEqual(result.remote_addr, changed_remote_addr)

    @override_settings(AUDIT_CLIENT_IP_RESOLVER_FUNCTION=custom_client_ip_func)
    def test_get_custom_client_ip_path(self):
        self.apply_middleware()
        remote_addr = "192.168.1.1"
        changed_remote_addr = "192.168.2.1"
        self.request.META["REMOTE_ADDR"] = remote_addr
        result = AuditContext.from_request(self.request)
        self.assertEqual(result.remote_addr, changed_remote_addr)
